package dev.fitko.fitconnect.client.factory;

import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.api.config.Environment;
import dev.fitko.fitconnect.api.config.EnvironmentName;
import dev.fitko.fitconnect.api.config.SenderConfig;
import dev.fitko.fitconnect.api.exceptions.InitializationException;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.io.TempDir;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Objects;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.containsString;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.is;
import static org.junit.jupiter.api.Assertions.assertNotNull;

class ApplicationConfigLoaderTest {

    @Test
    void testLoadMissingConfigFile() {
        Assertions.assertThrows(InitializationException.class, () -> ApplicationConfigLoader.loadConfig("/no/path"));
    }

    @Test
    void testEnvironmentNotAvailable() throws IOException {

        // Given
        final String testConfigYaml = getResourceAsString("test-config-with-missing-env.yml");
        final ApplicationConfig config = ApplicationConfigLoader.loadConfig(testConfigYaml);

        //When
        final InitializationException exception = Assertions.assertThrowsExactly(InitializationException.class, () -> config.getCurrentEnvironment());

        //Then
        assertThat(exception.getMessage(), containsString("Available environments are: prod | staging | dev | testing | local"));
    }

    @Test
    void testLoadTestConfigFromString() throws IOException {

        // Given
        final String testConfigYaml = getResourceAsString("test-config.yml");

        final Environment devEnv = new Environment(
                "https://auth-testing.fit-connect.fitko.dev",
                "https://submission-api-testing.fit-connect.fitko.dev",
                true
        );

        final SenderConfig senderConfig = new SenderConfig("1", "123");

        // When
        final ApplicationConfig testConfig = ApplicationConfigLoader.loadConfigFromYaml(testConfigYaml);

        // Then
        assertNotNull(testConfig);
        assertThat(testConfig.getCurrentEnvironment(), equalTo(devEnv));
        assertThat(testConfig.getSenderConfig(), equalTo(senderConfig));
    }

    @Test
    void testLoadTestConfigFromPath(@TempDir final Path tempDir) throws IOException {

        // Given
        final String testConfigYaml = getResourceAsString("test-config.yml");

        final Path configPath = Files.writeString(Path.of(tempDir.toString(), "test-config.yml"), testConfigYaml);

        // When
        final ApplicationConfig testConfig = ApplicationConfigLoader.loadConfig(configPath);

        // Then
        assertNotNull(testConfig);
        assertThat(testConfig.getActiveEnvironment(), is(new EnvironmentName("dev")));
    }

    private String getResourceAsString(final String name) throws IOException {
        final ClassLoader classLoader = getClass().getClassLoader();
        final File file = new File(Objects.requireNonNull(classLoader.getResource(name)).getFile());
        return Files.readString(file.toPath());
    }
}